package jp.co.ecosysnetwork.ccloca_app.service;

import android.annotation.SuppressLint;
import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.os.IBinder;
import android.os.Looper;

import androidx.core.app.NotificationCompat;

import com.google.android.gms.location.*;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.ccloca_app.R;
import jp.co.ecosysnetwork.ccloca_app.models.busRoute_info;
import jp.co.ecosysnetwork.ccloca_app.models.course_settings;
import jp.co.ecosysnetwork.ccloca_app.models.drive_state;
import jp.co.ecosysnetwork.ccloca_app.models.location_send;
import jp.co.ecosysnetwork.ccloca_app.models.system_settings;
import jp.co.ecosysnetwork.ccloca_app.thread.CommThread;
import jp.co.ecosysnetwork.universal.EsnDebugLog;

/**
 * 運転サービス
 */
public class DriveService extends Service {
    //region 定数
    /**
     * 通知チャネルID
     */
    public static final String CHANNEL_ID = "100";
    /**
     * 運転サービスID
     */
    private static final int SERVICE_ID = 9999;
    /**
     * 位置情報を採用する最小精度
     */
    private static final float MIN_ACC = 100;
    private static final long MIN_CYCLE_MSEC_LOCATION_CONTROL = 3000;
    /**
     * 初期位置情報取得サイクル
     */
    private static final long INIT_MSEC_LOCATION_INTERVAL = 1000;
    /**
     * 初期位置情報取得サイクル
     */
    private static final long INIT_MSEC_LOCATION_FAST_INTERVAL = 1000;
    //endregion
    //region メンバ変数
    /**
     * シングルトンのアプリケーション参照
     */
    private final MyApplication _app = MyApplication.getInstance();
    /**
     * システム設定モデル
     */
    private system_settings _system_settings;
    /**
     * コース設定モデル
     */
    private course_settings _course_settings;
    /**
     * 位置情報クライアント
     */
    private FusedLocationProviderClient _fusedLocationProviderClient;
    /**
     * 位置情報コールバックハンドラ
     */
    private LocationCallback _locationCallback;
    /**
     * サービスが開始されているか否か
     */
    private static boolean _isStarted = false;
    /**
     * 位置情報取得中であるか否か
     */
    private boolean _isActiveUpdateLocation = false;
    /**
     * 前回の位置制御実施時間
     */
    private long _preTimeLocationControl = 0;
//    /**
//     * サーバ通信サービスが停止していると判断する、サーバとの不通時間
//     */
//    private static final int DISABLE_MSEC_COMM = 30 * 1000;
    //endregion
    //region 内部メソッド
    /**
     * 位置情報による制御
     * @param lat 緯度
     * @param lon 経度
     * @param acc 精度
     * @param alt 標高
     */
    private void locationControl(double lat, double lon, double acc, double alt) {
        //region 位置制御を実施する時間であるか判定
        long nowTime = System.currentTimeMillis();
        if ((nowTime - _preTimeLocationControl) < MIN_CYCLE_MSEC_LOCATION_CONTROL) {
            //EsnDebugLog.outputLog(String.format(Locale.JAPAN, "規定間隔未満であるため位置制御実施しない now=%d pre=%d", nowTime, _preTimeLocationControl), EsnDebugLog.DIV.INFO);
            return;
        }
        _preTimeLocationControl = nowTime;
        //endregion

        // 携帯端末IDが未設定の場合は処理しない
        if ((_system_settings.phone_id == null) || (_system_settings.phone_id.length() == 0)) {
            EsnDebugLog.outputLog("携帯端末IDが未設定のため運転制御しない", EsnDebugLog.DIV.ERR);
            return;
        }

        //region 最初の位置情報受信時、オープニング音声を再生する
        if (!_app.drive_state_model.getLocationRecvedFirst()) {
            EsnDebugLog.outputLog("最初の受信", EsnDebugLog.DIV.INFO);
            _app.drive_state_model.setLocationRecvedFirst(true);
            _app.playWav(R.raw.opening, 1000);
        }
        //endregion

        //region 現在のコースIDを設定
        busRoute_info busRoute_info = _course_settings.selected_busRoute_info;
        if (busRoute_info.bus_route_no.equals("0")) {
            EsnDebugLog.outputLog("現在のコースIDが未設定", EsnDebugLog.DIV.ERR);
            return;
        }
        //endregion

        //region 運転中以外は以降処理しない
        drive_state.OPERATION_STATE operation_state = _app.drive_state_model.getOperationState();
        if (operation_state == drive_state.OPERATION_STATE.INACTIVE) {//region 運行外
            // 運転中以外は以降処理しない
            EsnDebugLog.outputLog("運行外時に処理が行われました", EsnDebugLog.DIV.ERR);
            return;
            //endregion
        }
            location_send sendLocation = new location_send();

            // 日替わり時、運行状態をクリアする
            SimpleDateFormat fmtYYYYMMDD = new SimpleDateFormat("yyyy-MM-dd", Locale.JAPANESE);
            String sendDate = fmtYYYYMMDD.format(new Date());
            SimpleDateFormat fmtHHiiss = new SimpleDateFormat("HH:mm:ss", Locale.JAPANESE);
            String sendTime = fmtHHiiss.format(new Date());
            // 位置情報をサーバに送信要求
            sendLocation.phone_no = _system_settings.phone_id;
            sendLocation.bus_route_no = _course_settings.selected_busRoute_info.bus_route_no;
            sendLocation.latitude = lat;
            sendLocation.longitude = lon;
            sendLocation.date = sendDate + sendTime;

//            // CommServiceを起動し、サーバ送信データリストを引き渡す
//            Intent intent = new Intent();
//            intent.putExtra(CommService.EXTRA_POSITION_SENDS, sendList);
//            CommService.enqueueWork(MyApplication.getInstance(), intent);

            // CommThreadを起動し、サーバにデータ送信
            CommThread th = new CommThread(_system_settings.server_url, sendLocation);
            th.start();
        //endregion
    }
    /**
     * 位置情報取得の開始
     * @param interval 位置情報の更新間間隔（単位：ミリ秒）
     * @param fastestInterval 位置情報の最短更新間隔（単位：ミリ秒）
     */
    @SuppressLint("MissingPermission")
    private void startLocationUpdates(long interval, long fastestInterval) {
        // 位置情報取得中の場合は一旦停止する
        if (_isActiveUpdateLocation) {
            stopLocationUpdates();
        }
        // 位置情報の取得を開始する
        LocationRequest locReq = LocationRequest.create();
        locReq.setInterval(interval);
        locReq.setFastestInterval(fastestInterval);
        locReq.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        _fusedLocationProviderClient.requestLocationUpdates(locReq, _locationCallback, Looper.getMainLooper());
        _isActiveUpdateLocation = true;

        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "位置情報の取得開始 interval=%d fastestInterval=%d", interval, fastestInterval), EsnDebugLog.DIV.INFO);
    }
    /**
     * 位置情報取得の停止
     */
    private void stopLocationUpdates() {
        if (_isActiveUpdateLocation) {
            EsnDebugLog.outputLog("位置情報の更新を停止", EsnDebugLog.DIV.INFO);
            _fusedLocationProviderClient.removeLocationUpdates(_locationCallback);
            _isActiveUpdateLocation = false;
        }
        else {
            EsnDebugLog.outputLog("位置情報の更新は既に停止している", EsnDebugLog.DIV.WARN);
        }
    }
    //endregion
    //region イベントメソッド
    /**
     * サービス作成イベント
     */
    @Override
    public void onCreate() {
        EsnDebugLog.outputLog("運転サービス onCreate", EsnDebugLog.DIV.INFO);
        super.onCreate();

        // メンバ変数設定
        _system_settings = new system_settings();
        _course_settings = new course_settings();

        // 位置情報クライアント生成
        _fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(this);
    }
    /**
     * サービスの開始イベント
     * @param intent インテント
     * @param flags フラグ
     * @param startId 開始ID
     * @return  START_STICKY固定
     */
    @SuppressLint("WakelockTimeout")
    @Override
    public int onStartCommand(final Intent intent, int flags, int startId) {
        EsnDebugLog.outputLog("運転サービス onStartCommand", EsnDebugLog.DIV.INFO);

        // サービス開始フラグON
        _isStarted = true;

        // ロケーションコールバック定義
        _locationCallback = new LocationCallback() {
            /**
             * ロケーション取得結果
             *
             * @param locationResult 位置情報の取得結果
             */
            @Override
            public void onLocationResult(LocationResult locationResult) {
                if (locationResult != null) {
                    for (Location loc : locationResult.getLocations()) {
                        double lat = loc.getLatitude();     // 緯度
                        double lon = loc.getLongitude();    // 経度
                        float acc = loc.getAccuracy();      // 精度
                        double alt = loc.getAltitude();      // 標高
                        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "lat=%f lon=%f acc=%f alt=%f", lat, lon, acc, alt), EsnDebugLog.DIV.INFO);

                        _app.drive_state_model.setLastLocationActiveMsec(System.currentTimeMillis());

                        // 取得値が最低精度を満たしている場合、位置情報制御を実施する
                        if ((acc < MIN_ACC) && (0 < lat) && (0 < lon)) {
                            locationControl(lat, lon, acc, alt);
                        }
                    }
                }
                super.onLocationResult(locationResult);
            }
        };

        // 通知インテントの作成
        PendingIntent openIntent = PendingIntent.getActivity(this, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        Notification notification = new NotificationCompat.Builder(this, CHANNEL_ID)
                .setSmallIcon(R.drawable.ic_launcher_foreground)
                .setContentTitle(MyApplication.getInstance().getString(R.string.driveNotifyName2))
                .setContentText(MyApplication.getInstance().getString(R.string.driveNotifyDescription2))
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                .setContentIntent(openIntent)
                .build();

        // サービスのフォアグラウンド化
        startForeground(SERVICE_ID, notification);

        // 位置情報取得開始
        startLocationUpdates(_system_settings.update_cycle_msec, _system_settings.update_cycle_msec);

        return START_STICKY;
    }
    /**
     * サービスのバインドイベント
     * @param intent インテント
     * @return null固定
     */
    @Override
    public IBinder onBind(Intent intent) {
        EsnDebugLog.outputLog("運転サービス onBind", EsnDebugLog.DIV.INFO);
        return null;
    }
    /**
     * サービスの廃棄イベント
     */
    @Override
    public void onDestroy() {
        EsnDebugLog.outputLog("運転サービス onDestroy", EsnDebugLog.DIV.INFO);
        super.onDestroy();

//        // 通信サービスの終了
//        CommService.requestStop();

        stopLocationUpdates();
        stopForeground(Service.STOP_FOREGROUND_DETACH);
        stopSelf();
        // サービス開始フラグOFF
        _isStarted = false;
    }
    //endregion
    //region 外部公開メソッド
    /**
     * サービスが開始されているか否かを返す
     * @return サービスが開始されているか否か
     */
    public static boolean isStarted() {
        return _isStarted;
    }
    //endregion
}
