package jp.co.ecosysnetwork.universal;

import android.os.Environment;
import android.util.Log;

import java.io.File;
import java.io.FileWriter;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.Locale;

/**
 * ECOSYSNETWORK K.K. デバッグログ出力クラス
 */
public class EsnDebugLog {
    //region 列挙体

    /**
     * ログ区分
     */
    public enum DIV {
        /**
         * デバッグ
         */
        DBG,
        /**
         * 情報
         */
        INFO,
        /**
         * 警告
         */
        WARN,
        /**
         * エラー
         */
        ERR
    }
    //endregion
    //region 定数
    /**
     * 日付部のフォーマット。拡張子の前に設定する。
     */
    private static final String DATE_FORMAT = "yyyyMMdd";
    //endregion
    //region 内部変数
    /**
     * 排他制御用オブジェクト
     */
    private static final Object _lockObj = new Object();
    /**
     * ログファイル名（ディレクトリ名。末尾に\をつける）
     */
    private static String _logFileNameDir;
    /**
     * ログファイル名（拡張子前）
     */
    private static String _logFileNameBody;
    /**
     * ログファイル名（拡張子。.を含む）
     */
    private static String _logFileNameExt;
    /**
     * ログファイル保持日数
     */
    private static int _logKeepDays;
    /**
     * 出力するログレベル（設定された値以上のログを出力する）
     */
    private static DIV _logLevel;
    //endregion
    //region 内部メソッド

    /**
     * 保持日数を過ぎたログファイルを削除する
     */
    private static void cleanUpLog() {
        synchronized (_lockObj) {
            try {
                File lfd = new File(_logFileNameDir);
                if (lfd.exists()) {
                    // ログファイル一覧を取得
                    String searchPtn = _logFileNameBody + "*" + _logFileNameExt;
                    EsnFileSearch efs = new EsnFileSearch();
                    File[] pathes = efs.listFiles(_logFileNameDir, searchPtn);

                    if (pathes != null) {
                        // 現在日からログファイルを保持する年月日を取得
                        Calendar cld = Calendar.getInstance();
                        cld.setTime(new Date());
                        cld.add(Calendar.DATE, -_logKeepDays);
                        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT, Locale.JAPAN);
                        String limitDate = sdf.format(cld.getTime());

                        // ログファイル一覧から年月日部を抽出し、保持する年月日より古いファイルは削除する
                        int datePos = _logFileNameBody.length();
                        for (File path : pathes) {
                            String pathName = path.getName();
                            if (pathName.length() >= (datePos + DATE_FORMAT.length())) {
                                String dateStr = pathName.substring(datePos, datePos + DATE_FORMAT.length());
                                if (dateStr.compareTo(limitDate) < 0) {
                                    if (!path.delete()) {
                                        Log.e(EsnUtility.getClassName(3) + EsnUtility.getMethodName(3), "delete failure." + pathName);
                                    }
                                }
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
    //endregion
    //region 外部公開メソッド

    /**
     * 初期化処理
     *
     * @param dirName     ログファイルのディレクトリ（末尾にフォルダマークつけること）
     * @param fileName    ログファイル名
     * @param logKeepDays ログファイルを保持する日数
     * @param logLevel    出力するログレベル（設定された値以上のログを出力する）
     */
    public static void init(String dirName, String fileName, int logKeepDays, DIV logLevel) {
        // ログファイル情報設定
        _logFileNameDir = dirName;
        _logFileNameBody = EsnUtility.getFilePreffix(fileName);
        _logFileNameExt = "." + EsnUtility.getFileSuffix(fileName);

        // ログファイルを保持する日数
        _logKeepDays = logKeepDays;

        // ログレベル
        _logLevel = logLevel;

        // 保持期間を過ぎたログファイルの削除
        cleanUpLog();
    }

    /**
     * ログ出力処理
     *
     * @param msg   ログメッセージ
     * @param div   ログ区分
     * @param title ログタイトル
     */
    private static void outputLog(String msg, DIV div, String title) {
        // 出力するログレベルでない場合は処理しない
        if (div.ordinal() < _logLevel.ordinal()) {
            return;
        }

        synchronized (_lockObj) {
            try {
                // ログディレクトリがない場合、作成する
                File lfd = new File(_logFileNameDir);
                if (!lfd.exists()) {
                    if (!lfd.mkdirs()) {
                        Log.e(EsnUtility.getClassName(3) + EsnUtility.getMethodName(3), "mkdirs error." + _logFileNameDir);
                        return;
                    }
                }

                // ログファイルフルパス生成
                Date nowDate = new Date();
                SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT, Locale.JAPAN);
                String path = _logFileNameDir + _logFileNameBody + sdf.format(nowDate.getTime()) + _logFileNameExt;

                // ログメッセージ作成
                StringBuilder sb = new StringBuilder();
                sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", Locale.JAPAN);
                sb.append(sdf.format(nowDate.getTime()));
                switch (div) {
                    case DBG:
                        sb.append("【D】");
                        break;
                    case INFO:
                        sb.append("【I】");
                        break;
                    case WARN:
                        sb.append("【W】");
                        break;
                    case ERR:
                        sb.append("【E】");
                        break;
                }
                sb.append("[");
                sb.append(title);
                sb.append("]");
                sb.append(msg);
                sb.append("\r\n");

                // ログファイル出力
                FileWriter fw = new FileWriter(path, true);
                fw.write(sb.toString());
                fw.close();

                // アウトプットウィンドウへの出力
                Log.d(EsnUtility.getClassName(3) + EsnUtility.getMethodName(3), sb.toString());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * デバッグログ出力処理
     *
     * @param logMsg ログメッセージ
     * @param logDiv ログ区分
     */
    static public void outputLog(String logMsg, DIV logDiv) {
        // タイトルの生成
        String title = EsnUtility.getClassName(4) + "-" + EsnUtility.getMethodName(4);
        outputLog(logMsg, logDiv, title);
    }

    /**
     * ログファイル一覧取得
     *
     * @return list ログファイル一覧
     */
    static public String[] GetLogFileList() {
        String[] list = null;
        try {
            File lfd = new File(_logFileNameDir);
            list = lfd.list();
        } catch (Exception ex) {
            EsnDebugLog.outputLog("ログファイル一覧取得失敗", EsnDebugLog.DIV.ERR);
        }
        return list;
    }

    /**
     * ログファイルディレクトリの取得
     *
     * @return LOGDIR ログファイルディレクトリ
     */
    static public String GetLogDir() {
        return _logFileNameDir;
    }
    //endregion
}
