package jp.co.ecosysnetwork.universal;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TreeSet;

/**
 * ECOSYSNETWORK K.K. ファイル検索クラス
 */
public class EsnFileSearch {
    //region 列挙体

    /**
     * 検索タイプ
     */
    public enum TYPE {
        /**
         * ファイル、ディレクトリ両方
         */
        BOTH,
        /**
         * ファイル
         */
        FILE,
        /**
         * ディレクトリ
         */
        DIR
    }
    //endregion
    //region メンバ変数
    /**
     * アルファベット順に並べるためTreeSetを使用。
     */
    private final TreeSet<File> _treeSet = new TreeSet<>();
    //endregion
    //region 内部メソッド

    /**
     * 指定条件に該当するファイル、ディレクトリを格納する
     *
     * @param type   検索タイプ
     * @param match  マッチングパターン
     * @param set    条件に該当するファイル、ディレクトリの格納際
     * @param file   検査対象のファイル/ディレクトリ
     * @param period ファイル/ディレクトリ更新後の経過日数
     */
    private void addFile(TYPE type, String match, TreeSet<File> set, File file, int period) {
        // 指定タイプでない場合抜ける
        if (type == TYPE.FILE) {
            if (!file.isFile()) {
                return;
            }
        } else if (type == TYPE.DIR) {
            if (!file.isDirectory()) {
                return;
            }
        }
        if (match != null && !file.getName().matches(match)) {
            return;
        }
        // 指定日数経過しているかどうかの指定がある場合
        if (period != 0) {
            // ファイル更新日付
            Date lastModifiedDate = new Date(file.lastModified());
            String lastModifiedDateStr = new SimpleDateFormat("yyyyMMdd", Locale.JAPAN).format(lastModifiedDate);

            // 指定の日付（１日をミリ秒で計算）
            long oneDayTime = 24L * 60L * 60L * 1000L;
            long periodTime = oneDayTime * Math.abs(period);
            Date designatedDate = new Date(System.currentTimeMillis() - periodTime);
            String designatedDateStr = new SimpleDateFormat("yyyyMMdd", Locale.JAPAN).format(designatedDate);
            if (period > 0) {
                if (lastModifiedDateStr.compareTo(designatedDateStr) < 0) {
                    return;
                }
            } else {
                if (lastModifiedDateStr.compareTo(designatedDateStr) > 0) {
                    return;
                }
            }
        }
        // 全ての条件に該当する場合リストに格納
        set.add(file);
    }
    //endregion
    //region 外部公開メソッド

    /**
     * インスタンスを生成後、続けて使用する場合は、このメソッドを
     * 呼び出しクリアする必要がある。
     */
    public void clear() {
        _treeSet.clear();
    }

    /**
     * 指定したディレクトリ[directoryPath]から、
     * 検索対象のファイル[fileName]を再帰的に検索し、該当する
     * ファイルオブジェクトのリストを返します。
     *
     * @param directoryPath 検索対象のディレクトリを表すパス
     * @param fileName      検索対象のファイル名
     *                      ファイル名にはワイルドカード文字として*を指定可能
     * @return 検索にマッチしたファイルオブジェクト
     */
    public File[] listFiles(String directoryPath, String fileName) {
        // ワイルドカード文字として*を正規表現に変換
        if (fileName != null) {
            fileName = fileName.replace(".", "\\.");
            fileName = fileName.replace("*", ".*");
        }
        return listFiles(directoryPath, fileName, TYPE.FILE, true, 0);
    }

    /**
     * 指定したディレクトリ[directoryPath]から、正規表現として指定された
     * 検索対象のファイル[fileNamePattern]を再帰的に検索し、
     * 該当するファイルオブジェクトのリストを返します。
     * <p>
     * また、ファイルの更新日付が指定日数経過しているかどうかを検索条件に
     * 指定する事ができます。
     *
     * @param directoryPath   検索対象のディレクトリを表すパス
     * @param fileNamePattern 検索対象のファイル名[正規表現]
     * @param type            該当するファイルオブジェクトの指定
     * @param isRecursive     再帰的に検索する場合はtrue
     * @param period          検索対象として、ファイルの更新日付が指定日数経過
     *                        しているかどうかを設定可能
     *                        0の場合は対象外
     *                        1以上の場合、指定日数以降のファイルを検索対象とする
     *                        -1以下の場合、指定日数以前のファイルを検索対象とする
     * @return 検索にマッチしたファイルオブジェクト
     */
    private File[] listFiles(String directoryPath, String fileNamePattern, TYPE type, boolean isRecursive, int period) {

        File dir = new File(directoryPath);
        if (!dir.isDirectory()) {
            throw new IllegalArgumentException("引数で指定されたパス[" + dir.getAbsolutePath() + "]はディレクトリではありません。");
        }
        File[] files = dir.listFiles();
        // その出力
        for (File file : files) {
            addFile(type, fileNamePattern, _treeSet, file, period);
            // 再帰的に検索＆ディレクトリならば再帰的にリストに追加
            if (isRecursive && file.isDirectory()) {
                listFiles(file.getAbsolutePath(), fileNamePattern, type, isRecursive, period);
            }
        }
        return _treeSet.toArray(new File[_treeSet.size()]);
    }
    //endregion
}
